/*
    SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
    SPDX-FileCopyrightText: 2021-2022 Harald Sitter <sitter@kde.org>
*/

import QtQuick 2.15
import QtQuick.Controls 2.5 as QQC2
import QtQuick.Layouts 1.1

import org.kde.kirigami 2.19 as Kirigami
import org.kde.kcm 1.6 as KCM

import org.kde.kinfocenter.about_astra.private 1.0

KCM.SimpleKCM {
    id: root

    // bugger off with all your buttons, we don't need any!
    KCM.ConfigModule.buttons: KCM.ConfigModule.NoAdditionalButton

    implicitWidth: !kcm.isFlat ? Kirigami.Units.gridUnit * 20 : gridLayout.implicitWidth + Kirigami.Units.largeSpacing
    implicitHeight: !kcm.isFlat ? Kirigami.Units.gridUnit * 20 : gridLayout.implicitHeight + Kirigami.Units.largeSpacing

    // Scoot away from the edges! Otherwise the KCM looks overly cramped.
    topPadding: !kcm.isFlat ? Kirigami.Units.largeSpacing * 4 : 0
    leftPadding: Kirigami.Units.largeSpacing * 4
    rightPadding: Kirigami.Units.largeSpacing * 4
    bottomPadding: !kcm.isFlat ? Kirigami.Units.largeSpacing * 4 : 0

    GridLayout {
        id: gridLayout

        columnSpacing: Kirigami.Units.largeSpacing
        rowSpacing: Kirigami.Units.largeSpacing

        columns: kcm.isFlat ? 2 : 1

        RowLayout {
            Layout.alignment: Qt.AlignHCenter
            spacing: Kirigami.Units.largeSpacing

            ColumnLayout {
                Item { Layout.fillHeight: true }

                Kirigami.Icon {
                    id: distroIcon
                    source: kcm.distroLogo
                    implicitWidth: 145
                    implicitHeight: 80
                }

                Kirigami.Heading {
                    text: kcm.distroNameVersion
                    level: 1
                    type: Kirigami.Heading.Type.Primary
                }

                Kirigami.Heading {
                    visible: kcm.securityLevel !== ""
                    text: kcm.securityLevel
                    level: 2
                    type: Kirigami.Heading.Type.Secondary
                }

                Kirigami.Heading {
                    visible: kcm.distroVariant !== ""
                    text: kcm.distroVariant
                    level: 2
                    type: Kirigami.Heading.Type.Secondary
                }

                QQC2.Label {
                    visible: kcm.distroUrl !== ""
                    text: "<a href='%1'>%1</a>".arg(kcm.distroUrl)
                    textFormat: Text.RichText
                    onLinkActivated: Qt.openUrlExternally(link)

                    HoverHandler { cursorShape: Qt.PointingHandCursor }
                }

                Item { Layout.fillHeight: true }
            }
        }

        Kirigami.FormLayout {
            Layout.fillHeight: true
            wideMode: true

            Component {
                id: entryComponent
                RowLayout {
                    Kirigami.FormData.label: modelData.localizedLabel()
                    readonly property bool hidden: modelData.isHidden()

                    Component {
                        id: unhideDialog
                        Kirigami.PromptDialog {
                            // NB: should we ever have other entries that need this dialog then this needs refactoring on the Entry side.
                            //  Do NOT simply add if else logic here!
                            title: i18nc("@title", "Serial Number")
                            subtitle: modelData.localizedValue()
                            flatFooterButtons: true
                            standardButtons: Kirigami.Dialog.NoButton
                            customFooterActions: [
                                Kirigami.Action {
                                    text: i18nc("@action:button", "Copy to Clipboard")
                                    icon.name: "edit-copy"
                                    onTriggered: kcm.storeInClipboard(subtitle)
                                    shortcut: StandardKey.Copy
                                }
                            ]
                        }
                    }

                    QQC2.Label {
                        visible: !hidden
                        text: modelData.localizedValue()
                    }
                    QQC2.Button {
                        visible: hidden
                        text: i18nc("@action:button show a hidden entry in an overlay", "Show")
                        onClicked: {
                            const dialog = unhideDialog.createObject(root, {})
                            dialog.open()
                        }
                    }
                }
            }

            Item {
                visible: kcm.isFlat
                implicitHeight: distroIcon.implicitHeight
            }

            // TODO: the spacers were added for 5.23 because we can't fix Kirigami in time. See if later versions
            // are neat enough without the spacers. Also see https://invent.kde.org/frameworks/kirigami/-/merge_requests/357
            // Same for the spacer below.
            //Item { implicitHeight: Kirigami.Units.gridUnit }
            Repeater {
                model: kcm.softwareEntries
                delegate: entryComponent
            }

            // TODO: same as for the spacer above
            Item {
                visible: !kcm.isNoHardwareEntries
                implicitHeight: Kirigami.Units.gridUnit
            }
            Kirigami.Separator {
                visible: !kcm.isNoHardwareEntries
                Kirigami.FormData.label: i18nc("@title:group", "Hardware")
                Kirigami.FormData.isSection: true
            }
            Repeater {
                visible: !kcm.isNoHardwareEntries
                model: kcm.hardwareEntries
                delegate: entryComponent
            }
        }

        ColumnLayout { // Wraps a spacer and button (conditionally visible!)
            Layout.alignment: Qt.AlignHCenter
            visible: !kcm.isThisKInfoCenter && kicRunner.canRun

            Item { implicitHeight: !kcm.isFlat ? Kirigami.Units.gridUnit : 0 }
            QQC2.Button {
                id: kicButton

                text: i18nc("@action:button launches kinfocenter from systemsettings", "Show More Information")
                icon.name: kicRunner.iconName
                onClicked: kicRunner.run()
            }

            ServiceRunner {
                id: kicRunner
                desktopFileName: "org.kde.kinfocenter"
            }
        }
    }

    footer: QQC2.Control { // wrapping the footer to get control-style padding
        visible: !kcm.isFlat

        contentItem: Kirigami.ActionToolBar {
            flat: false

            actions: [
                Kirigami.Action {
                    text: i18nc("@action:button", "Copy to Clipboard")
                    icon.name: "edit-copy"
                    onTriggered: kcm.copyToClipboard()
                    shortcut: StandardKey.Copy
                },
                Kirigami.Action {
                    visible: false
                    text: i18nc("@action:button", "Copy to Clipboard in English")
                    icon.name: "edit-copy"
                    onTriggered: kcm.copyToClipboardInEnglish()
                }
            ]
        }
    }

    Component.onCompleted: {
        if (kcm.isFlat) {
            root.globalToolBarStyle = Kirigami.ApplicationHeaderStyle.None;
        }
    }
}
